@@ -842,6 +842,17 @@ impl GemView {
                             .expect("Cannot send data");
                         break;
                     }
+                    gemini::protocol::StatusCode::Input(sensitive) => {
+                        let input = gemini::Input {
+                            meta: response.meta,
+                            url: url.to_string(),
+                            sensitive,
+                        };
+                        sender
+                            .send(gemini::Response::RequestInput(input))
+                            .expect("Cannot send data");
+                        break;
+                    }
                     s => {
                         let estr = format!("{:?}", s);
                         sender
@@ -855,6 +866,15 @@ impl GemView {
         let viewer = self.clone();
         receiver.attach(None, move |response| {
             match response {
+                gemini::Response::RequestInput(input) => {
+                    let signal = if input.sensitive == 1 {
+                        "request-input-sensitive"
+                    } else {
+                        "request-input"
+                    };
+                    viewer.append_history(&input.url);
+                    viewer.emit_by_name::<()>(signal, &[&input.meta, &input.url]);
+                }
                 gemini::Response::Success(content) => {
                     viewer.set_buffer_mime(&content.mime);
                     viewer.set_buffer_content(&content.bytes);
@@ -990,6 +1010,22 @@ impl GemView {
         })
     }
 
+    /// Connects to the "request-input" signal, emitted when the server has
+    /// requested input. The signal handler should repeat the page request with
+    /// the user input appended.
+    pub fn connect_request_input<F: Fn(&Self, String, String) + 'static>(
+        &self,
+        f: F,
+    ) -> glib::SignalHandlerId {
+        self.connect_local("request-input", true, move |values| {
+            let obj = values[0].get::<Self>().unwrap();
+            let meta = values[1].get::<String>().unwrap();
+            let url = values[2].get::<String>().unwrap();
+            f(&obj, meta, url);
+            None
+        })
+    }
+
     fn wrap_text(&self, text: &str, font_size: i32) -> String {
         let factor = font_size / 1500;
         let width: usize = match self.root() {
